@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  $Id: pstdout.c,v 1.10 2010-02-10 01:27:44 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007-2013 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2014 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -126,7 +126,9 @@ static pthread_mutex_t pstdout_launch_mutex = PTHREAD_MUTEX_INITIALIZER;
 static List pstdout_states = NULL;
 static pthread_mutex_t pstdout_states_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+#ifndef HAVE_SIGHANDLER_T
 typedef void (*sighandler_t)(int);
+#endif /* HAVE_SIGHANDLER_T */
 
 static struct pstdout_consolidated_data *
 _pstdout_consolidated_data_create(const char *hostname, const char *output)
