@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 FreeIPMI Core Team
+ * Copyright (C) 2003-2014 FreeIPMI Core Team
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include <time.h>
 #endif /* !HAVE_SYS_TIME_H */
 #endif  /* !TIME_WITH_SYS_TIME */
+#include <assert.h>
 
 #include "freeipmi/api/ipmi-messaging-support-cmds-api.h"
 #include "freeipmi/api/ipmi-device-global-cmds-api.h"
@@ -1089,14 +1090,83 @@ ipmi_cmd_set_system_info_parameters_set_in_progress (ipmi_ctx_t ctx,
   return (rv);
 }
 
-int
-ipmi_cmd_set_system_info_parameters_system_firmware_version_first_set (ipmi_ctx_t ctx,
-                                                                       uint8_t set_selector,
-                                                                       uint8_t encoding,
-                                                                       uint8_t string_length,
-                                                                       const void *string_block,
-                                                                       unsigned int string_block_length,
-                                                                       fiid_obj_t obj_cmd_rs)
+static int
+_ipmi_cmd_set_system_info_parameters_string_first_set_common (ipmi_ctx_t ctx,
+							      uint8_t set_selector,
+							      uint8_t encoding,
+							      uint8_t string_length,
+							      const void *string_block,
+							      unsigned int string_block_length,
+							      fiid_obj_t obj_cmd_rs,
+							      fiid_field_t *tmpl_cmd_rq,
+							      int (*fill_func)(uint8_t, uint8_t, uint8_t, const void *, unsigned int, fiid_obj_t))
+{
+  fiid_obj_t obj_cmd_rq = NULL;
+  int rv = -1;
+  
+  assert (tmpl_cmd_rq);
+  assert (fill_func);
+
+  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  /* remaining parameter checks in fill function */
+  if (!fiid_obj_valid (obj_cmd_rs))
+    {
+      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
+      return (-1);
+    }
+								      
+  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs, tmpl_cmd_set_system_info_parameters_rs) < 0)
+    {
+      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
+      return (-1);
+    }
+
+  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_rq)))
+    {
+      API_ERRNO_TO_API_ERRNUM (ctx, errno);
+      goto cleanup;
+    }
+
+  if (fill_func (set_selector,
+		 encoding,
+		 string_length,
+		 string_block,
+		 string_block_length,
+		 obj_cmd_rq) < 0)
+    {
+      API_ERRNO_TO_API_ERRNUM (ctx, errno);
+      goto cleanup;
+    }
+  
+  if (api_ipmi_cmd (ctx,
+                    IPMI_BMC_IPMB_LUN_BMC,
+                    IPMI_NET_FN_APP_RQ,
+                    obj_cmd_rq,
+                    obj_cmd_rs) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      goto cleanup;
+    }
+  
+  rv = 0;
+ cleanup:
+  fiid_obj_destroy (obj_cmd_rq);
+  return (rv);
+}
+
+static int
+_ipmi_cmd_set_system_info_parameters_string_set_common (ipmi_ctx_t ctx,
+							uint8_t set_selector,
+							const void *string_block,
+							unsigned int string_block_length,
+							fiid_obj_t obj_cmd_rs,
+							fiid_field_t *tmpl_cmd_rq,
+							int (*fill_func)(uint8_t, const void *, unsigned int, fiid_obj_t))
 {
   fiid_obj_t obj_cmd_rq = NULL;
   int rv = -1;
@@ -1114,25 +1184,22 @@ ipmi_cmd_set_system_info_parameters_system_firmware_version_first_set (ipmi_ctx_
       return (-1);
     }
 
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
+  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs, tmpl_cmd_set_system_info_parameters_rs) < 0)
     {
       API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
       return (-1);
     }
 
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_system_firmware_version_first_set_rq)))
+  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_rq)))
     {
       API_ERRNO_TO_API_ERRNUM (ctx, errno);
       goto cleanup;
     }
 
-  if (fill_cmd_set_system_info_parameters_system_firmware_version_first_set (set_selector,
-                                                                             encoding,
-                                                                             string_length,
-                                                                             string_block,
-                                                                             string_block_length,
-                                                                             obj_cmd_rq) < 0)
+  if (fill_func (set_selector,
+		 string_block,
+		 string_block_length,
+		 obj_cmd_rq) < 0)
     {
       API_ERRNO_TO_API_ERRNUM (ctx, errno);
       goto cleanup;
@@ -1154,6 +1221,32 @@ ipmi_cmd_set_system_info_parameters_system_firmware_version_first_set (ipmi_ctx_
   return (rv);
 }
 
+int
+ipmi_cmd_set_system_info_parameters_system_firmware_version_first_set (ipmi_ctx_t ctx,
+                                                                       uint8_t set_selector,
+                                                                       uint8_t encoding,
+                                                                       uint8_t string_length,
+                                                                       const void *string_block,
+                                                                       unsigned int string_block_length,
+                                                                       fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_set_system_info_parameters_string_first_set_common (ctx,
+								    set_selector,
+								    encoding,
+								    string_length,
+								    string_block,
+								    string_block_length,
+								    obj_cmd_rs,
+								    tmpl_cmd_set_system_info_parameters_system_firmware_version_first_set_rq,
+								    fill_cmd_set_system_info_parameters_system_firmware_version_first_set) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
 int
 ipmi_cmd_set_system_info_parameters_system_firmware_version (ipmi_ctx_t ctx,
                                                              uint8_t set_selector,
@@ -1161,58 +1254,19 @@ ipmi_cmd_set_system_info_parameters_system_firmware_version (ipmi_ctx_t ctx,
                                                              unsigned int string_block_length,
                                                              fiid_obj_t obj_cmd_rs)
 {
-  fiid_obj_t obj_cmd_rq = NULL;
-  int rv = -1;
-
-  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+  if (_ipmi_cmd_set_system_info_parameters_string_set_common (ctx,
+							      set_selector,
+							      string_block,
+							      string_block_length,
+							      obj_cmd_rs,
+							      tmpl_cmd_set_system_info_parameters_system_firmware_version_rq,
+							      fill_cmd_set_system_info_parameters_system_firmware_version) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
       return (-1);
     }
 
-  /* remaining parameter checks in fill function */
-  if (!fiid_obj_valid (obj_cmd_rs))
-    {
-      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
-      return (-1);
-    }
-
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
-    {
-      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
-      return (-1);
-    }
-
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_system_firmware_version_rq)))
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (fill_cmd_set_system_info_parameters_system_firmware_version (set_selector,
-                                                                   string_block,
-                                                                   string_block_length,
-                                                                   obj_cmd_rq) < 0)
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (api_ipmi_cmd (ctx,
-                    IPMI_BMC_IPMB_LUN_BMC,
-                    IPMI_NET_FN_APP_RQ,
-                    obj_cmd_rq,
-                    obj_cmd_rs) < 0)
-    {
-      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
-      goto cleanup;
-    }
-
-  rv = 0;
- cleanup:
-  fiid_obj_destroy (obj_cmd_rq);
-  return (rv);
+  return (0);
 }
 
 int
@@ -1224,60 +1278,21 @@ ipmi_cmd_set_system_info_parameters_system_name_first_set (ipmi_ctx_t ctx,
                                                            unsigned int string_block_length,
                                                            fiid_obj_t obj_cmd_rs)
 {
-  fiid_obj_t obj_cmd_rq = NULL;
-  int rv = -1;
-
-  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+  if (_ipmi_cmd_set_system_info_parameters_string_first_set_common (ctx,
+								    set_selector,
+								    encoding,
+								    string_length,
+								    string_block,
+								    string_block_length,
+								    obj_cmd_rs,
+								    tmpl_cmd_set_system_info_parameters_system_name_first_set_rq,
+								    fill_cmd_set_system_info_parameters_system_name_first_set) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
       return (-1);
     }
 
-  /* remaining parameter checks in fill function */
-  if (!fiid_obj_valid (obj_cmd_rs))
-    {
-      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
-      return (-1);
-    }
-
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
-    {
-      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
-      return (-1);
-    }
-
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_system_name_first_set_rq)))
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (fill_cmd_set_system_info_parameters_system_name_first_set (set_selector,
-                                                                 encoding,
-                                                                 string_length,
-                                                                 string_block,
-                                                                 string_block_length,
-                                                                 obj_cmd_rq) < 0)
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (api_ipmi_cmd (ctx,
-                    IPMI_BMC_IPMB_LUN_BMC,
-                    IPMI_NET_FN_APP_RQ,
-                    obj_cmd_rq,
-                    obj_cmd_rs) < 0)
-    {
-      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
-      goto cleanup;
-    }
-
-  rv = 0;
- cleanup:
-  fiid_obj_destroy (obj_cmd_rq);
-  return (rv);
+  return (0);
 }
 
 int
@@ -1287,58 +1302,19 @@ ipmi_cmd_set_system_info_parameters_system_name (ipmi_ctx_t ctx,
                                                  unsigned int string_block_length,
                                                  fiid_obj_t obj_cmd_rs)
 {
-  fiid_obj_t obj_cmd_rq = NULL;
-  int rv = -1;
-
-  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+  if (_ipmi_cmd_set_system_info_parameters_string_set_common (ctx,
+							      set_selector,
+							      string_block,
+							      string_block_length,
+							      obj_cmd_rs,
+							      tmpl_cmd_set_system_info_parameters_system_name_rq,
+							      fill_cmd_set_system_info_parameters_system_name) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
       return (-1);
     }
 
-  /* remaining parameter checks in fill function */
-  if (!fiid_obj_valid (obj_cmd_rs))
-    {
-      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
-      return (-1);
-    }
-
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
-    {
-      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
-      return (-1);
-    }
-
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_system_name_rq)))
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (fill_cmd_set_system_info_parameters_system_name (set_selector,
-                                                       string_block,
-                                                       string_block_length,
-                                                       obj_cmd_rq) < 0)
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (api_ipmi_cmd (ctx,
-                    IPMI_BMC_IPMB_LUN_BMC,
-                    IPMI_NET_FN_APP_RQ,
-                    obj_cmd_rq,
-                    obj_cmd_rs) < 0)
-    {
-      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
-      goto cleanup;
-    }
-
-  rv = 0;
- cleanup:
-  fiid_obj_destroy (obj_cmd_rq);
-  return (rv);
+  return (0);
 }
 
 int
@@ -1350,60 +1326,21 @@ ipmi_cmd_set_system_info_parameters_primary_operating_system_name_first_set (ipm
                                                                              unsigned int string_block_length,
                                                                              fiid_obj_t obj_cmd_rs)
 {
-  fiid_obj_t obj_cmd_rq = NULL;
-  int rv = -1;
-
-  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+  if (_ipmi_cmd_set_system_info_parameters_string_first_set_common (ctx,
+								    set_selector,
+								    encoding,
+								    string_length,
+								    string_block,
+								    string_block_length,
+								    obj_cmd_rs,
+								    tmpl_cmd_set_system_info_parameters_primary_operating_system_name_first_set_rq,
+								    fill_cmd_set_system_info_parameters_primary_operating_system_name_first_set) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
       return (-1);
     }
 
-  /* remaining parameter checks in fill function */
-  if (!fiid_obj_valid (obj_cmd_rs))
-    {
-      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
-      return (-1);
-    }
-
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
-    {
-      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
-      return (-1);
-    }
-
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_primary_operating_system_name_first_set_rq)))
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (fill_cmd_set_system_info_parameters_primary_operating_system_name_first_set (set_selector,
-                                                                                   encoding,
-                                                                                   string_length,
-                                                                                   string_block,
-                                                                                   string_block_length,
-                                                                                   obj_cmd_rq) < 0)
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (api_ipmi_cmd (ctx,
-                    IPMI_BMC_IPMB_LUN_BMC,
-                    IPMI_NET_FN_APP_RQ,
-                    obj_cmd_rq,
-                    obj_cmd_rs) < 0)
-    {
-      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
-      goto cleanup;
-    }
-
-  rv = 0;
- cleanup:
-  fiid_obj_destroy (obj_cmd_rq);
-  return (rv);
+  return (0);
 }
 
 int
@@ -1413,58 +1350,19 @@ ipmi_cmd_set_system_info_parameters_primary_operating_system_name (ipmi_ctx_t ct
                                                                    unsigned int string_block_length,
                                                                    fiid_obj_t obj_cmd_rs)
 {
-  fiid_obj_t obj_cmd_rq = NULL;
-  int rv = -1;
-
-  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+  if (_ipmi_cmd_set_system_info_parameters_string_set_common (ctx,
+							      set_selector,
+							      string_block,
+							      string_block_length,
+							      obj_cmd_rs,
+							      tmpl_cmd_set_system_info_parameters_primary_operating_system_name_rq,
+							      fill_cmd_set_system_info_parameters_primary_operating_system_name) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
       return (-1);
     }
 
-  /* remaining parameter checks in fill function */
-  if (!fiid_obj_valid (obj_cmd_rs))
-    {
-      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
-      return (-1);
-    }
-
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
-    {
-      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
-      return (-1);
-    }
-
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_primary_operating_system_name_rq)))
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (fill_cmd_set_system_info_parameters_primary_operating_system_name (set_selector,
-                                                                         string_block,
-                                                                         string_block_length,
-                                                                         obj_cmd_rq) < 0)
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (api_ipmi_cmd (ctx,
-                    IPMI_BMC_IPMB_LUN_BMC,
-                    IPMI_NET_FN_APP_RQ,
-                    obj_cmd_rq,
-                    obj_cmd_rs) < 0)
-    {
-      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
-      goto cleanup;
-    }
-
-  rv = 0;
- cleanup:
-  fiid_obj_destroy (obj_cmd_rq);
-  return (rv);
+  return (0);
 }
 
 int
@@ -1476,60 +1374,21 @@ ipmi_cmd_set_system_info_parameters_operating_system_name_first_set (ipmi_ctx_t
                                                                      unsigned int string_block_length,
                                                                      fiid_obj_t obj_cmd_rs)
 {
-  fiid_obj_t obj_cmd_rq = NULL;
-  int rv = -1;
-
-  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+  if (_ipmi_cmd_set_system_info_parameters_string_first_set_common (ctx,
+								    set_selector,
+								    encoding,
+								    string_length,
+								    string_block,
+								    string_block_length,
+								    obj_cmd_rs,
+								    tmpl_cmd_set_system_info_parameters_operating_system_name_first_set_rq,
+								    fill_cmd_set_system_info_parameters_operating_system_name_first_set) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
       return (-1);
     }
 
-  /* remaining parameter checks in fill function */
-  if (!fiid_obj_valid (obj_cmd_rs))
-    {
-      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
-      return (-1);
-    }
-
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
-    {
-      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
-      return (-1);
-    }
-
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_operating_system_name_first_set_rq)))
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (fill_cmd_set_system_info_parameters_operating_system_name_first_set (set_selector,
-                                                                           encoding,
-                                                                           string_length,
-                                                                           string_block,
-                                                                           string_block_length,
-                                                                           obj_cmd_rq) < 0)
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (api_ipmi_cmd (ctx,
-                    IPMI_BMC_IPMB_LUN_BMC,
-                    IPMI_NET_FN_APP_RQ,
-                    obj_cmd_rq,
-                    obj_cmd_rs) < 0)
-    {
-      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
-      goto cleanup;
-    }
-
-  rv = 0;
- cleanup:
-  fiid_obj_destroy (obj_cmd_rq);
-  return (rv);
+  return (0);
 }
 
 int
@@ -1539,58 +1398,163 @@ ipmi_cmd_set_system_info_parameters_operating_system_name (ipmi_ctx_t ctx,
                                                            unsigned int string_block_length,
                                                            fiid_obj_t obj_cmd_rs)
 {
-  fiid_obj_t obj_cmd_rq = NULL;
-  int rv = -1;
-
-  if (!ctx || ctx->magic != IPMI_CTX_MAGIC)
+  if (_ipmi_cmd_set_system_info_parameters_string_set_common (ctx,
+							      set_selector,
+							      string_block,
+							      string_block_length,
+							      obj_cmd_rs,
+							      tmpl_cmd_set_system_info_parameters_operating_system_name_rq,
+							      fill_cmd_set_system_info_parameters_operating_system_name) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
       return (-1);
     }
 
-  /* remaining parameter checks in fill function */
-  if (!fiid_obj_valid (obj_cmd_rs))
-    {
-      API_SET_ERRNUM (ctx, IPMI_ERR_PARAMETERS);
-      return (-1);
-    }
+  return (0);
+}
 
-  if (FIID_OBJ_TEMPLATE_COMPARE (obj_cmd_rs,
-                                 tmpl_cmd_set_system_info_parameters_rs) < 0)
-    {
-      API_FIID_OBJECT_ERROR_TO_API_ERRNUM (ctx, obj_cmd_rs);
-      return (-1);
-    }
-
-  if (!(obj_cmd_rq = fiid_obj_create (tmpl_cmd_set_system_info_parameters_operating_system_name_rq)))
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (fill_cmd_set_system_info_parameters_operating_system_name (set_selector,
-                                                                 string_block,
-                                                                 string_block_length,
-                                                                 obj_cmd_rq) < 0)
-    {
-      API_ERRNO_TO_API_ERRNUM (ctx, errno);
-      goto cleanup;
-    }
-
-  if (api_ipmi_cmd (ctx,
-                    IPMI_BMC_IPMB_LUN_BMC,
-                    IPMI_NET_FN_APP_RQ,
-                    obj_cmd_rq,
-                    obj_cmd_rs) < 0)
+int
+ipmi_cmd_set_system_info_parameters_present_os_version_number_first_set (ipmi_ctx_t ctx,
+									 uint8_t set_selector,
+									 uint8_t encoding,
+									 uint8_t string_length,
+									 const void *string_block,
+									 unsigned int string_block_length,
+									 fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_set_system_info_parameters_string_first_set_common (ctx,
+								    set_selector,
+								    encoding,
+								    string_length,
+								    string_block,
+								    string_block_length,
+								    obj_cmd_rs,
+								    tmpl_cmd_set_system_info_parameters_present_os_version_number_first_set_rq,
+								    fill_cmd_set_system_info_parameters_present_os_version_number_first_set) < 0)
     {
       ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
-      goto cleanup;
+      return (-1);
     }
 
-  rv = 0;
- cleanup:
-  fiid_obj_destroy (obj_cmd_rq);
-  return (rv);
+  return (0);
+}
+
+int
+ipmi_cmd_set_system_info_parameters_present_os_version_number (ipmi_ctx_t ctx,
+							       uint8_t set_selector,
+							       const void *string_block,
+							       unsigned int string_block_length,
+							       fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_set_system_info_parameters_string_set_common (ctx,
+							      set_selector,
+							      string_block,
+							      string_block_length,
+							      obj_cmd_rs,
+							      tmpl_cmd_set_system_info_parameters_present_os_version_number_rq,
+							      fill_cmd_set_system_info_parameters_present_os_version_number) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_set_system_info_parameters_bmc_url_first_set (ipmi_ctx_t ctx,
+						       uint8_t set_selector,
+						       uint8_t encoding,
+						       uint8_t string_length,
+						       const void *string_block,
+						       unsigned int string_block_length,
+						       fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_set_system_info_parameters_string_first_set_common (ctx,
+								    set_selector,
+								    encoding,
+								    string_length,
+								    string_block,
+								    string_block_length,
+								    obj_cmd_rs,
+								    tmpl_cmd_set_system_info_parameters_bmc_url_first_set_rq,
+								    fill_cmd_set_system_info_parameters_bmc_url_first_set) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_set_system_info_parameters_bmc_url (ipmi_ctx_t ctx,
+					     uint8_t set_selector,
+					     const void *string_block,
+					     unsigned int string_block_length,
+					     fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_set_system_info_parameters_string_set_common (ctx,
+							      set_selector,
+							      string_block,
+							      string_block_length,
+							      obj_cmd_rs,
+							      tmpl_cmd_set_system_info_parameters_bmc_url_rq,
+							      fill_cmd_set_system_info_parameters_bmc_url) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_set_system_info_parameters_base_os_hypervisor_url_first_set (ipmi_ctx_t ctx,
+								      uint8_t set_selector,
+								      uint8_t encoding,
+								      uint8_t string_length,
+								      const void *string_block,
+								      unsigned int string_block_length,
+								      fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_set_system_info_parameters_string_first_set_common (ctx,
+								    set_selector,
+								    encoding,
+								    string_length,
+								    string_block,
+								    string_block_length,
+								    obj_cmd_rs,
+								    tmpl_cmd_set_system_info_parameters_base_os_hypervisor_url_first_set_rq,
+								    fill_cmd_set_system_info_parameters_base_os_hypervisor_url_first_set) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_set_system_info_parameters_base_os_hypervisor_url (ipmi_ctx_t ctx,
+							    uint8_t set_selector,
+							    const void *string_block,
+							    unsigned int string_block_length,
+							    fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_set_system_info_parameters_string_set_common (ctx,
+							      set_selector,
+							      string_block,
+							      string_block_length,
+							      obj_cmd_rs,
+							      tmpl_cmd_set_system_info_parameters_base_os_hypervisor_url_rq,
+							      fill_cmd_set_system_info_parameters_base_os_hypervisor_url) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
 }
 
 static int
@@ -1877,6 +1841,138 @@ ipmi_cmd_get_system_info_parameters_operating_system_name (ipmi_ctx_t ctx,
   return (0);
 }
 
+int
+ipmi_cmd_get_system_info_parameters_present_os_version_number_first_set (ipmi_ctx_t ctx,
+									 uint8_t get_parameter,
+									 uint8_t set_selector,
+									 uint8_t block_selector,
+									 fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_get_system_info_parameters_common (ctx,
+						   get_parameter,
+						   set_selector,
+						   block_selector,
+						   obj_cmd_rs,
+						   tmpl_cmd_get_system_info_parameters_present_os_version_number_first_set_rs,
+						   IPMI_SYSTEM_INFO_PARAMETER_PRESENT_OS_VERSION_NUMBER) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_get_system_info_parameters_present_os_version_number (ipmi_ctx_t ctx,
+							       uint8_t get_parameter,
+							       uint8_t set_selector,
+							       uint8_t block_selector,
+							       fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_get_system_info_parameters_common (ctx,
+						   get_parameter,
+						   set_selector,
+						   block_selector,
+						   obj_cmd_rs,
+						   tmpl_cmd_get_system_info_parameters_present_os_version_number_rs,
+						   IPMI_SYSTEM_INFO_PARAMETER_PRESENT_OS_VERSION_NUMBER) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_get_system_info_parameters_bmc_url_first_set (ipmi_ctx_t ctx,
+						       uint8_t get_parameter,
+						       uint8_t set_selector,
+						       uint8_t block_selector,
+						       fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_get_system_info_parameters_common (ctx,
+						   get_parameter,
+						   set_selector,
+						   block_selector,
+						   obj_cmd_rs,
+						   tmpl_cmd_get_system_info_parameters_bmc_url_first_set_rs,
+						   IPMI_SYSTEM_INFO_PARAMETER_BMC_URL) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_get_system_info_parameters_bmc_url (ipmi_ctx_t ctx,
+					     uint8_t get_parameter,
+					     uint8_t set_selector,
+					     uint8_t block_selector,
+					     fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_get_system_info_parameters_common (ctx,
+						   get_parameter,
+						   set_selector,
+						   block_selector,
+						   obj_cmd_rs,
+						   tmpl_cmd_get_system_info_parameters_bmc_url_rs,
+						   IPMI_SYSTEM_INFO_PARAMETER_BMC_URL) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_get_system_info_parameters_base_os_hypervisor_url_first_set (ipmi_ctx_t ctx,
+								      uint8_t get_parameter,
+								      uint8_t set_selector,
+								      uint8_t block_selector,
+								      fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_get_system_info_parameters_common (ctx,
+						   get_parameter,
+						   set_selector,
+						   block_selector,
+						   obj_cmd_rs,
+						   tmpl_cmd_get_system_info_parameters_base_os_hypervisor_url_first_set_rs,
+						   IPMI_SYSTEM_INFO_PARAMETER_BASE_OS_HYPERVISOR_URL) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
+int
+ipmi_cmd_get_system_info_parameters_base_os_hypervisor_url (ipmi_ctx_t ctx,
+							    uint8_t get_parameter,
+							    uint8_t set_selector,
+							    uint8_t block_selector,
+							    fiid_obj_t obj_cmd_rs)
+{
+  if (_ipmi_cmd_get_system_info_parameters_common (ctx,
+						   get_parameter,
+						   set_selector,
+						   block_selector,
+						   obj_cmd_rs,
+						   tmpl_cmd_get_system_info_parameters_base_os_hypervisor_url_rs,
+						   IPMI_SYSTEM_INFO_PARAMETER_BASE_OS_HYPERVISOR_URL) < 0)
+    {
+      ERR_TRACE (ipmi_ctx_errormsg (ctx), ipmi_ctx_errnum (ctx));
+      return (-1);
+    }
+
+  return (0);
+}
+
 int
 ipmi_cmd_get_session_challenge (ipmi_ctx_t ctx,
                                 uint8_t authentication_type,
