@@ -233,6 +233,8 @@ extern "C" {
 #define IPMI_SUPERMICRO_PRODUCT_ID_X8SIE           1037
 #define IPMI_SUPERMICRO_PRODUCT_ID_X9SCA_F_O       1585
 #define IPMI_SUPERMICRO_PRODUCT_ID_H8DGU_F         43025
+#define IPMI_SUPERMICRO_PRODUCT_ID_H8DG6           48145
+#define IPMI_SUPERMICRO_PRODUCT_ID_H8DGU           4520
 #define IPMI_SUPERMICRO_PRODUCT_ID_X9DRI_LN4F_PLUS 1574
 
 /* Seen in the wild w/ proper IANA number 10876 */
